#pragma once

#include <future>
#include <utility>
#include "execution_defs.h"
#include "execution_manager.h"
#include "executor_abstract.h"
#include "index/ix.h"
#include "system/sm.h"

class LoadExecutor : public AbstractExecutor {
private:
    TabMeta tab_;
    std::string file_name_;
    std::string table_name_;
    SmManager *sm_manager_;
    RmFileHandle *fh_;
    Rid rid_;

public:
    LoadExecutor(SmManager *sm_manager, std::string file_name, const std::string &table_name, Context *context)
            : file_name_(std::move(file_name)), table_name_(table_name), sm_manager_(sm_manager) {
        fh_ = sm_manager_->fhs_.at(table_name).get();
        tab_ = sm_manager_->db_.get_table(table_name);
        context_ = context;
    }

    std::unique_ptr<RmRecord> Next() override {
        (*context_->load_count)++;

        // Launching the task asynchronously
        std::async(std::launch::async, [this]() {
            auto start_time = time(nullptr);
            sm_manager_->load_csv_itermodel(file_name_, table_name_);
            (*context_->load_count)--;
            std::cout << "load " << table_name_ << " insert complete!" << std::endl;
            auto end_time = time(nullptr);
            std::cout << "load using " << end_time - start_time << " seconds\n";
        });

        std::cout << "load ok!" << std::endl;

        // Returning an empty std::unique_ptr as actual return value is handled asynchronously
        return nullptr;
    }

    Rid &rid() override { return rid_; }
};